/*
 * $Id: cwsutil.c,v 1.2 2010/08/05 22:41:53 tyreld Exp $
 *
 * (C) Copyright IBM Corp. 2003, 2005, 2010
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE ECLIPSE PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Eclipse Public License from
 * http://www.opensource.org/licenses/eclipse-1.0.php
 *
 * Author:      Viktor Mihajlovski <mihajlov@de.ibm.com>
 *
 * Description: Utility functions for CMPI Workshop
 * 
 */

#include "cwsutil.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

typedef struct {
  FILE *fp;
  char name[L_tmpnam];
} CWS_Control;

void* CWS_Begin_Enum(const char *topdir, int filetype, int recursive)
{
  /* begin enumeration */
  char cmdbuffer[2000];
  CWS_Control *cc = malloc(sizeof(CWS_Control));
  if (cc && tmpnam(cc->name)) {
    sprintf(cmdbuffer,
	    "find %s -follow -type %c -not -path %s %s"
	    "-printf \"%%p %%s " 
	    "%%C@ %%A@ %%T@ %%m\n\" > %s",
	    topdir, filetype, topdir, recursive ? "":"-maxdepth 1 ",cc->name);
    if (system(cmdbuffer)==0)
      cc->fp = fopen(cc->name,"r");
    else {
      free(cc);
      cc=NULL;
    }
  }
  return cc;
}

int CWS_Next_Enum(void *handle, CWS_FILE* cwsf)
{
  /* read next entry from result file */
  char result[2000];
  CWS_Control *cc = (CWS_Control *)handle;
  int state=0;
  if (cc && cwsf && fgets(result,sizeof(result),cc->fp))
    state=0<sscanf(result,"%s %lld %ld %ld %ld %o",
		   cwsf->cws_name,
		   &cwsf->cws_size,
		   &cwsf->cws_ctime,
		   &cwsf->cws_mtime,
		   &cwsf->cws_atime,
		   &cwsf->cws_mode);
  return state;
}

void CWS_End_Enum(void *handle)
{
  /* end enumeration */
  CWS_Control *cc = (CWS_Control *)handle;
  if (cc) {
    fclose(cc->fp);
    remove(cc->name);
    free(cc);
  }
}

int CWS_Get_File(const char *file, CWS_FILE* cwsf)
{
  int         state=0;
  struct stat statbuf; 
  if (file && cwsf && stat(file,&statbuf)==0) {
    strcpy(cwsf->cws_name,file);
    cwsf->cws_size=statbuf.st_size;
    cwsf->cws_ctime=statbuf.st_ctime;
    cwsf->cws_mtime=statbuf.st_mtime;
    cwsf->cws_atime=statbuf.st_atime;
    cwsf->cws_mode=statbuf.st_mode;
    state=1;
  }
  return state;
}

int CWS_Update_File(CWS_FILE* cwsf)
{

  int         state=0;
  struct stat statbuf; 
  int         mode;

  /* only change mode */
  if (cwsf && stat(cwsf->cws_name,&statbuf)==0) {
    mode=(statbuf.st_mode & 07077) | (cwsf->cws_mode & 0700);
    state=chmod(cwsf->cws_name,mode)==0;
  }
  return state;
}

int CWS_Create_Directory(CWS_FILE* cwsf)
{

  int         state=0;
  int         mode;

  if (cwsf) {
    mode=cwsf->cws_mode | 0077; /* plus umask */
    state=mkdir(cwsf->cws_name,mode)==0;
  }
  return state;
}

int CWS_Get_FileType(const char *file, char* typestring, size_t tslen)
{
  char  cmdbuffer[400];
  char  cmdout[300];
  FILE *fcmdout;
  
  if (file && tmpnam(cmdout)) {
    sprintf(cmdbuffer,"file %s > %s\n",file,cmdout);
    if (system(cmdbuffer)==0 && 
	(fcmdout = fopen(cmdout,"r")) &&
	fgets(typestring,tslen,fcmdout)) {
      fclose(fcmdout);
      return 0;
  }
  }
  return 1;
}
